# -*- coding: utf-8 -*-
import os
import click

from .base import Penv


abspath = os.path.abspath
override_cd_bash_function = """
function cd () {
    builtin cd "$@" && eval "$(penv scan)"
}
"""


@click.group(invoke_without_command=True)
@click.option('--override-cd-bash', default=False, is_flag=True,
              help='Just prints the command to be evaluated by bash.')
@click.pass_context
def cli(ctx, override_cd_bash):
    if ctx.invoked_subcommand is None and override_cd_bash:
        return click.echo(override_cd_bash_function)

    if ctx.invoked_subcommand is None:
        return click.echo(ctx.command.get_help(ctx))

    # Maybe I'll make "place" customizable at some point
    ctx.obj = {
        'place': abspath('.'),
        'override_cd_bash': override_cd_bash,
    }


# $> penv [--place=/path/to/place/] scan
@cli.command('scan')
@click.pass_context
def cli_stan(ctx, env=Penv()):
    place = ctx.obj['place']
    return click.echo(env.lookup(place))



# -*- coding: utf-8 -*-
"""
Small helpers that makes easy to generate bash.
"""
import pipes
from cStringIO import StringIO


class BashStream(object):
    def __init__(self):
        self.out = StringIO()

    def getvalue(self):
        return self.out.getvalue()

    def writeln(self, text):
        self.out.write(text)
        self.out.write("\n")


class Bash(object):
    def echo(self, message):
        return "; ".join([
            'echo %s' % pipes.quote(msg)
            for msg in message.split("\n")
        ])

    def export(self, name, value):
        return 'export %s=%s' % (name, value)

    def source(self, file_path):
        return 'source %s' % file_path

    def unset(self, name):
        return 'unset %s' % name

    def unset_f(self, name):
        return 'unset -f %s' % name

    def function(self, name, definition):
        return 'function %s () { %s; }' % (name, definition)


class BashMixin(object):
    bash = Bash()



# -*- coding: utf-8 -*-
import io
import re
from setuptools import setup, find_packages


version = re.search(
    r"VERSION = ['\"](.*)['\"]",
    io.open('penv/__init__.py').read()
).group(1)


setup(
    name='penv',
    version=version,
    description='Framework for ease project management tools building.',
    author='Jakub Janoszek',
    author_email='kuba.janoszek@gmail.com',
    include_package_data=True,
    url='https://github.com/jqb/penv/tree/ver-%s' % version,
    packages=find_packages(),
    install_requires=[
        "click==3.3",
    ],
    entry_points=io.open('entry-points.ini', 'r').read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
    zip_safe=False,
)

# Usage of setup.py:
# $> python setup.py register                         # registering package on PYPI
# $> python setup.py build sdist bdist_wheel upload   # build, make source dist and upload to PYPI



# -*- coding: utf-8 -*-
import os


exists = os.path.exists
pjoin = os.path.join
expanduser = os.path.expanduser


class Configuration(object):
    def __init__(self, data):
        self.__dict__.update(**data)


class ConfigurationReader(object):
    DEFAULT_TRIGGER = '.penv'
    DEFAULT_SKIP_FILE = '.skip'

    def read_defaults(self, config):
        config.update({
            'TRIGGER': os.environ.get('PENV_TRIGGER', self.DEFAULT_TRIGGER),
            'PLUGIN_PLACES': [
                pjoin(expanduser("~"), '.penv', '.plugins'),
            ],
            'SKIP_FILE': os.environ.get('PENV_SKIP_FILE', self.DEFAULT_SKIP_FILE),
        })
        return config

    def read_local_penv(self, config, root_new):
        plugins = config.get('PLUGIN_PLACES')

        # Some default, current-dir-location-related, places where I'd
        # like to find plugins. The idea is that ".penv" is NOT
        # under version control but ".penv-plugins" might be.
        plugins.append(root_new('.penv', '.plugins'))
        plugins.append(root_new('.penv-plugins'))

        skip_file_path = root_new(config.get('TRIGGER'),
                                  config.get('SKIP_FILE'))
        skip_file_exists = exists(skip_file_path)
        config.update({
            'PLUGIN_PLACES': plugins,
            'SKIP_FILE_PATH': skip_file_path,
            'SKIP_FILE_EXISTS': skip_file_exists,
        })
        return config

    def read(self, config, root_new):
        config = self.read_defaults(config)
        config = self.read_local_penv(config, root_new)
        return Configuration(config)


class ConfigurationMixin(object):
    configuration_reader = ConfigurationReader()

    def get_config(self, root_new):
        return self.configuration_reader.read({}, root_new)
